@@ -1,89 +1,48 @@
-from enum import Enum
+import logging
 from typing import Optional, Dict
 
 import requests
 
-from .exceptions import (
-    BinanceChainAPIException, BinanceChainRequestException
+import binance_chain.messages
+from binance_chain.environment import BinanceEnvironment
+from binance_chain.constants import PeerType, KlineInterval, OrderSide, OrderStatus, TransactionSide, TransactionType
+from binance_chain.exceptions import (
+    BinanceChainAPIException, BinanceChainRequestException, BinanceChainBroadcastException
 )
 
 
-class KlineInterval(str, Enum):
-    ONE_MINUTE = '1m'
-    THREE_MINUTES = '3m'
-    FVE_MINUTES = '5m'
-    FIFTEEN_MINUTES = '15m'
-    THIRTY_MINUTES = '30m'
-    ONE_HOUR = '1h'
-    TWO_HOURS = '2h'
-    FOUR_HOURS = '4h'
-    SIX_HOURS = '6h'
-    EIGHT_HOURS = '8h'
-    TWELVE_HOURS = '12h'
-    ONE_DAY = '1d'
-    THREE_DAYS = '3d'
-    ONE_WEEK = '1w'
-    ONE_MONTH = '1M'
+class HttpApiClient:
 
-
-class OrderStatus(str, Enum):
-    ACK = 'Ack'
-    PARTIAL_FILL = 'PartialFill'
-    IOC_NO_FILL = 'IocNoFill'
-    FULLY_FILL = 'FullyFill'
-    CANCELED = 'Canceled'
-    EXPIRED = 'Expired'
-    FAILED_BLOCKING = 'FailedBlocking'
-    FAILED_MATCHING = 'FailedMatching'
-
-
-class OrderSide(str, Enum):
-    BUY = 'buy'
-    SELL = 'sell'
-
-
-class TransactionSide(str, Enum):
-    RECEIVE = 'RECEIVE'
-    SEND = 'SEND'
-
-
-class TransactionType(str, Enum):
-    NEW_ORDER = 'NEW_ORDER'
-    ISSUE_TOKEN = 'ISSUE_TOKEN'
-    BURN_TOKEN = 'BURN_TOKEN'
-    LIST_TOKEN = 'LIST_TOKEN'
-    CANCEL_ORDER = 'CANCEL_ORDER'
-    FREEZE_TOKEN = 'FREEZE_TOKEN'
-    UN_FREEZE_TOKEN = 'UN_FREEZE_TOKEN'
-    TRANSFER = 'TRANSFER'
-    PROPOSAL = 'PROPOSAL'
-    VOTE = 'VOTE'
-
-
-class Client(object):
-
-    TESTNET_API_URL = 'https://testnet-dex.binance.org'
     API_VERSION = 'v1'
 
-    def __init__(self, api_url: Optional[str] = None, requests_params: Optional[Dict] = None):
+    def __init__(self, env: Optional[BinanceEnvironment] = None, requests_params: Optional[Dict] = None):
         """Binance Chain API Client constructor
 
         https://binance-chain.github.io/api-reference/dex-api/paths.html
 
+        :param env: (optional) A BinanceEnvironment instance or None which will default to production env
         :param requests_params: (optional) Dictionary of requests params to use for all calls
         :type requests_params: dict.
 
         .. code:: python
 
-            client = Client(api_key, api_secret
+            # get production env client
+            prod_client = Client()
+
+            # get testnet env client
+            testnet_env = BinanceEnvironment.get_testnet_env()
+            client = Client(testnet_env)
 
         """
 
-        self.API_URL = api_url or self.TESTNET_API_URL
-
+        self._env = env or BinanceEnvironment.get_production_env()
         self._requests_params = requests_params
         self.session = self._init_session()
 
+    @property
+    def env(self):
+        return self._env
+
     def _init_session(self):
 
         session = requests.session()
@@ -98,7 +57,7 @@ class Client(object):
         return '/api/{}/{}'.format(self.API_VERSION, path)
 
     def _create_uri(self, path):
-        return '{}{}'.format(self.API_URL, path)
+        return '{}{}'.format(self._env.api_url, path)
 
     def _request(self, method, path, **kwargs):
 
@@ -114,11 +73,15 @@ class Client(object):
 
         full_path = self._create_path(path)
         uri = self._create_uri(full_path)
+        print(f"uri:{uri}")
 
         if kwargs['data'] and method == 'get':
             kwargs['params'] = kwargs['data']
             del(kwargs['data'])
 
+        if method == 'post':
+            kwargs['headers']['content-type'] = 'text/plain'
+
         response = getattr(self.session, method)(uri, **kwargs)
         return self._handle_response(response)
 
@@ -128,7 +91,6 @@ class Client(object):
         Raises the appropriate exceptions when necessary; otherwise, returns the
         response.
         """
-
         if not str(response.status_code).startswith('2'):
             raise BinanceChainAPIException(response)
         try:
@@ -209,7 +171,7 @@ class Client(object):
         """
         return self._get("validators")
 
-    def get_peers(self):
+    def get_peers(self, peer_type: Optional[PeerType] = None):
         """Gets the list of network peers
 
         https://binance-chain.github.io/api-reference/dex-api/paths.html#apiv1peers
@@ -221,7 +183,26 @@ class Client(object):
         :return: API Response
 
         """
-        return self._get("peers")
+        peers = self._get("peers")
+        if peer_type:
+            print(f'filtering peers for {peer_type}')
+            peers = [p for p in peers if peer_type in p['capabilities']]
+
+        return peers
+
+    def get_node_peers(self):
+        """Get a list of peers with Node RPC support
+
+        :return:
+        """
+        return self.get_peers(peer_type=PeerType.NODE)
+
+    def get_websocket_peers(self):
+        """Get a list of peers with Websocket support
+
+        :return:
+        """
+        return self.get_peers(peer_type=PeerType.WEBSOCKET)
 
     def get_account(self, address: str):
         """Gets account metadata for an address
@@ -333,27 +314,58 @@ class Client(object):
         }
         return self._get("depth", data=data)
 
-    def get_broadcast(self, transaction: str):
-        """Gets the order book depth data for a given pair symbol
+    def broadcast_msg(self, msg: binance_chain.messages.Msg, sync: bool = False):
+        """Broadcast a message
 
         https://binance-chain.github.io/api-reference/dex-api/paths.html#apiv1broadcast
 
-        :param symbol: required e.g NNB-0AD_BNB
+        :param msg: Type of NewOrderMsg, CancelOrderMsg, FreezeMsg, UnfreezeMsg
+        :param sync: Synchronous broadcast (wait for DeliverTx)?
 
         .. code:: python
 
-            order_book = client.get_order_book('NNB-0AD_BNB')
+            # new order example
+            # construct the message
+            new_order_msg = NewOrderMsg(
+                symbol="ANN-457_BNB",
+                time_in_force=TimeInForce.GTE,
+                order_type=OrderType.LIMIT,
+                side=OrderSide.BUY,
+                price=Decimal(0.000396000),
+                quantity=Decimal(12)
+            )
+            # then broadcast it
+            res = client.broadcast_msg(new_order_msg, wallet)
+
+            # cancel order example
+            cancel_order_msg = CancelOrderMsg(
+                order_id="09F8B32D33CBE2B546088620CBEBC1FF80F9BE001ACF42762B0BBFF0A729CE3",
+                symbol='ANN-457_BNB',
+            )
+            res = client.broadcast_msg(cancel_order_msg, wallet)
 
         :return: API Response
 
         """
-        # TODO: implement transactions
-        # data = {
-        #     'sync': sync
-        #     'body': body
-        # }
-        # return self._post("broadcast", False, data=data)
-        pass
+
+        # fetch account detail
+        # account = self.get_account(self.msg.wallet.address)
+
+        if self._env != msg.wallet.env:
+            raise BinanceChainBroadcastException("Wallet environment doesn't match HttpApiClient environment")
+
+        msg.wallet.initialise_wallet(self)
+        data = msg.to_hex_data()
+
+        logging.debug(f'data:{data}')
+
+        req_path = 'broadcast'
+        if sync:
+            req_path += f'?sync=1'
+
+        res = self._post(req_path, data=data)
+        msg.wallet.increment_account_sequence()
+        return res
 
     def get_klines(self, symbol: str, interval: KlineInterval, limit: Optional[int] = 300,
                    start_time: Optional[int] = None, end_time: Optional[int] = None):
